// database field base class                                     -*- C++ -*-
// $Id: field_base.cc,v 1.1 2004/05/22 17:58:07 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above
//    copyright notice, this list of conditions and the following
//    disclaimer in the documentation and/or other materials provided
//    with the distribution.
//  * Neither the name of the author, nor the names of other
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
//  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
//  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
//  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
//  SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

#include <pqxxobject/field_base.h>

using namespace pqxxobject;

field_base::field_base():
  m_column_name(),
  m_signal_changed()
{
}

field_base::field_base(const std::string& column_name):
  m_column_name(column_name),
  m_signal_changed()
{
}


field_base::field_base(const field_base& rhs):
  m_column_name(rhs.m_column_name),
  m_signal_changed()
{
}

field_base::~field_base()
{
}

field_base&
field_base::operator = (const field_base& rhs)
{
  // Don't assign the column name, since this could cause undesirable
  // side effects (differently-named columns could quite legitimately
  // have their values assigned to one another).
  return *this;
}

const std::string&
field_base::get_column_name() const
{
  return m_column_name;
}

void
field_base::set_column_name(const std::string& column_name)
{
  m_column_name = column_name;
}


SigC::Signal0<void>&
field_base::signal_changed()
{
  return m_signal_changed;
}

