// database insert query class                                   -*- C++ -*-
// $Id: insert_query.cc,v 1.1 2004/05/22 17:58:08 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above
//    copyright notice, this list of conditions and the following
//    disclaimer in the documentation and/or other materials provided
//    with the distribution.
//  * Neither the name of the author, nor the names of other
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
//  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
//  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
//  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
//  SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

#include <sstream>

#include <pqxxobject/insert_query.h>

using namespace pqxxobject;

insert_query::insert_query():
  pqxxobject::query(),
  table_name(),
  field_list()
{
}

insert_query::insert_query(const std::string& table):
  pqxxobject::query(),
  table_name(table),
  field_list()
{
}

insert_query::insert_query(const insert_query& rhs):
  pqxxobject::query(*this),
  table_name(rhs.table_name),
  field_list(rhs.field_list)
{
}

insert_query::~insert_query()
{
}

void
insert_query::remove(const pqxxobject::field_base& field)
{
  for (std::list<field_value>::iterator iter = field_list.begin();
       iter != field_list.end();
       ++iter)
    {
      if (iter->first == field.get_column_name())
	{
	  iter = field_list.erase(iter);
	}
    }
}

std::string insert_query::str() const
{
  // Erase the previous query (if any).
  std::ostringstream qs;
  // Construct the new query.
  if (field_list.size() > 0)
    {
      qs << "INSERT INTO " << table_name << '(';
      for (std::list<field_value>::const_iterator iter = field_list.begin();
	   iter != field_list.end();
	   ++iter)
	{
	  if (iter != field_list.begin())
	    qs << ", "; // For all but the first field.
	  qs << iter->first;
	}
      qs << ") VALUES (";
      for (std::list<field_value>::const_iterator iter = field_list.begin();
	   iter != field_list.end();
	   ++iter)
	{
	  if (iter != field_list.begin())
	    qs << ", "; // For all but the first field.
	  qs << iter->second;
	}
      qs << ')';
    }
  return qs.str();
}
