// database query helper functions                               -*- C++ -*-
// $Id: query_helpers.h,v 1.1 2004/05/22 17:58:09 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above
//    copyright notice, this list of conditions and the following
//    disclaimer in the documentation and/or other materials provided
//    with the distribution.
//  * Neither the name of the author, nor the names of other
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
//  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
//  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
//  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
//  SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

#ifndef PQXXOBJECT_QUERY_HELPERSH
#define PQXXOBJECT_QUERY_HELPERSH

#include <sstream>

namespace pqxxobject
{
  /**
   * Helper function used when constructing a query.  It handles
   * quoting and formatting of datatypes into a format suitable for an
   * SQL statement.
   */
  template< typename T, template <typename> class F >
  struct query_helper
  {
    /**
     * Construct a pair containing the field name and value from a field value.
     * @param field the field to use.
     * @returns a std::pair containing the field name and its value as
     * std::string types.
     */
    static std::pair< std::string, std::string >
    apply(F<T>& field)
    {
      std::ostringstream value;
      value << field;
      return std::pair< std::string, std::string >
	(field.get_column_name(), value.str());
    }
  };

  template< template <typename> class F >
  struct query_helper< std::string, F >
  {
    static std::pair< std::string, std::string >
    apply(F<std::string>& field)
    {
      std::ostringstream value;
      value << pqxx::Quote(*field);
      return std::pair< std::string, std::string >
	(field.get_column_name(), value.str());
    }
  };

  template< template <typename> class F >
  struct query_helper< bool, F >
  {
    static std::pair< std::string, std::string >
    apply(F<bool>& field)
    {
      std::ostringstream value;
      value << (*field ? "true" : "false");
      return std::pair< std::string, std::string >
	(field.get_column_name(), value.str());
    }
  };

}; // namespace pqxxobject

#endif // PQXXOBJECT_QUERY_HELPERSH
