// database update query class                                   -*- C++ -*-
// $Id: update_query.cc,v 1.1 2004/05/22 17:58:11 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above
//    copyright notice, this list of conditions and the following
//    disclaimer in the documentation and/or other materials provided
//    with the distribution.
//  * Neither the name of the author, nor the names of other
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
//  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
//  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
//  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
//  SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

#include <sstream>

#include <pqxxobject/update_query.h>

using namespace pqxxobject;

update_query::update_query():
  pqxxobject::query(),
  m_table_name(),
  m_field_list(),
  m_where()
{
}

update_query::update_query(const std::string& table):
  pqxxobject::query(),
  m_table_name(table),
  m_field_list(),
  m_where()
{
}

update_query::update_query(const update_query& rhs):
  pqxxobject::query(*this),
  m_table_name(rhs.m_table_name),
  m_field_list(rhs.m_field_list),
  m_where(rhs.m_where)
{
}

update_query::~update_query()
{
}

void
update_query::remove(const pqxxobject::field_base& field)
{
  for (std::list<field_value>::iterator iter = m_field_list.begin();
       iter != m_field_list.end();
       ++iter)
    {
      if (iter->first == field.get_column_name())
	{
	  iter = m_field_list.erase(iter);
	}
    }
}

#include <iostream>

std::string update_query::str() const
{
  // Erase the previous query (if any).
  std::ostringstream qs;
  // Construct the new query.
  if (m_field_list.size() > 0)
    {
      qs << "UPDATE " << m_table_name << " SET ";
      for (std::list<field_value>::const_iterator iter = m_field_list.begin();
	   iter != m_field_list.end();
	   ++iter)
	{
	  if (iter != m_field_list.begin())
	    qs << ", "; // For all but the first field.
	  qs << iter->first << " = " << iter->second;
	}
      if (m_where.length())
	qs << " WHERE (" << m_where << ')';
    }
  return qs.str();
}
